`timescale 1ns / 1ps

module clock_generator #(
	//f_clk_out = f_clk_in * F_CLKIN_MUL / F_CLKIN_DIV.
	//A bemen rajel frekvencija Hz-ben.
   parameter CLKIN_FREQ_HZ = 16000000.0,
	//Szorz a bemen rajel frekvencijhoz (2 - 32).
   parameter F_CLKIN_MUL   = 25,
	//Oszt a bemen rajel frekvencijhoz (1 - 32).
   parameter F_CLKIN_DIV   = 16
) (
	//Bemen rajel.
   input  wire clk_in,
	//Bemen reset jel.
   input  wire rst_in,
	//Kimen rajel.
   output wire clk_out,
	//Kimen reset jel.
   output wire rst_out
);

//*****************************************************************************
//* Digital Clock Manager (DCM) egysg pldnyostsa.                        *
//*****************************************************************************
localparam CLKIN_PERIOD_PS = 1000000000000.0 / CLKIN_FREQ_HZ;

wire dcm_clkin;
wire dcm_clk0;
wire dcm_clkfb;
wire dcm_clkfx;
wire dcm_locked;

DCM_SP #(
   .CLKDV_DIVIDE(2.0),                    // Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5,7.0
                                          //            7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
   .CLKFX_DIVIDE(F_CLKIN_DIV),            // Can be any integer from 1 to 32
   .CLKFX_MULTIPLY(F_CLKIN_MUL),          // Can be any integer from 2 to 32
   .CLKIN_DIVIDE_BY_2("FALSE"),           // TRUE/FALSE to enable CLKIN divide by two feature
   .CLKIN_PERIOD(CLKIN_PERIOD_PS),        // Specify period of input clock
   .CLKOUT_PHASE_SHIFT("NONE"),           // Specify phase shift of NONE, FIXED or VARIABLE
   .CLK_FEEDBACK("1X"),                   // Specify clock feedback of NONE, 1X or 2X
   .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),  // SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or an integer from 0 to 15
   .DLL_FREQUENCY_MODE("LOW"),            // HIGH or LOW frequency mode for DLL
   .DUTY_CYCLE_CORRECTION("TRUE"),        // Duty cycle correction, TRUE or FALSE
   .PHASE_SHIFT(0),                       // Amount of fixed phase shift from -255 to 255
   .STARTUP_WAIT("FALSE")                 // Delay configuration DONE until DCM LOCK, TRUE/FALSE
) DCM_SP_inst (
   .CLK0(dcm_clk0),                       // 0 degree DCM CLK output
   .CLK180(),                             // 180 degree DCM CLK output
   .CLK270(),                             // 270 degree DCM CLK output
   .CLK2X(),                              // 2X DCM CLK output
   .CLK2X180(),                           // 2X, 180 degree DCM CLK out
   .CLK90(),                              // 90 degree DCM CLK output
   .CLKDV(),                              // Divided DCM CLK out (CLKDV_DIVIDE)
   .CLKFX(dcm_clkfx),                     // DCM CLK synthesis out (M/D)
   .CLKFX180(),                           // 180 degree CLK synthesis out
   .LOCKED(dcm_locked),                   // DCM LOCK status output
   .PSDONE(),                             // Dynamic phase adjust done output
   .STATUS(),                             // 8-bit DCM status bits output
   .CLKFB(dcm_clkfb),                     // DCM clock feedback
   .CLKIN(dcm_clkin),                     // Clock input (from IBUFG, BUFG or DCM)
   .PSCLK(1'b0),                          // Dynamic phase adjust clock input
   .PSEN(1'b0),                           // Dynamic phase adjust enable input
   .PSINCDEC(1'b0),                       // Dynamic phase adjust increment/decrement
   .RST(rst_in)                           // DCM asynchronous reset input
);

//rajel pufferek.
BUFG clk_in_bufg(.I(clk_in), .O(dcm_clkin));
BUFG clk_out_bufg(.I(dcm_clkfx), .O(clk_out));
BUFG clk_fb_bufg(.I(dcm_clk0), .O(dcm_clkfb));


//*****************************************************************************
//* A kimen reset jel ellltsa.                                           *
//*****************************************************************************
reg [7:0] rst_sync_reg;

always @(posedge clk_out or negedge dcm_locked)
begin
	if (dcm_locked == 0)
		rst_sync_reg <= 8'b1111_1111;
	else
		rst_sync_reg <= {rst_sync_reg[6:0], 1'b0};
end

assign rst_out = rst_sync_reg[7];


endmodule
