`timescale 1ns / 1ps

//*****************************************************************************
//* Sakktbla minta generls.                                                *
//*****************************************************************************
module vga_chesspattern(
   input  wire        clk16M,
   input  wire        rstbt,
   output wire [12:5] aio
);

//******************************************************************************
//* rajel genertor. A 16 MHz bejv rajelbl ellltja a 25 MHz-es pixel   *
//* rajelet.                                                                  *
//******************************************************************************
wire clk;
wire rst;

clock_generator #(
	//f_clk_out = f_clk_in * F_CLKIN_MUL / F_CLKIN_DIV.
	//A bemen rajel frekvencija Hz-ben.
   .CLKIN_FREQ_HZ(16000000.0),
	//Szorz a bemen rajel frekvencijhoz (2 - 32).
   .F_CLKIN_MUL(25),
	//Oszt a bemen rajel frekvencijhoz (1 - 32).
   .F_CLKIN_DIV(16)
) clkgen (
	//Bemen rajel.
   .clk_in(clk16M),
	//Bemen reset jel.
   .rst_in(rstbt),
	//Kimen rajel.
   .clk_out(clk),
	//Kimen reset jel.
   .rst_out(rst)
);


//******************************************************************************
//* VGA idzts genertor 640x480 @ 60 Hz felbontshoz.                       *
//******************************************************************************
wire [9:0] h_cnt;
wire [9:0] v_cnt;
wire       blank;
wire       h_sync;
wire       v_sync;

vga_timing vga_timing(
   .clk(clk),           //25 MHz pixel rajel.
   .rst(rst),           //Reset jel.
   .h_cnt(h_cnt),       //Horizontlis szmll (X koordinta).
   .v_cnt(v_cnt),       //Vertiklis szmll (Y koordinta)
   .blank(blank),       //Kiolt jel.
   .h_sync(h_sync),     //Horizontlis szinkronjel.
   .v_sync(v_sync)      //Vertiklis szinkronjel.
);


//******************************************************************************
//* A kimenetek meghajtsa. Ha a sznjeleket regiszterezzk, akkor a szinkron  *
//* jeleket is ksleltetni kell 1 rajel peridussal. Ha a kiolt jel aktv,   *
//* akkor a sznjeleket 0-val hajtsuk meg (fekete szn).                       *
//******************************************************************************
reg [1:0] r_out, g_out, b_out;
reg       h_sync_out, v_sync_out;

always @(posedge clk)
begin
	r_out[1]   <= ...;
	r_out[0]   <= ...;
	g_out[1]   <= ...;
	g_out[0]   <= ...;
	b_out[1]   <= ...;
	b_out[0]   <= ...;
	h_sync_out <= h_sync;
	v_sync_out <= v_sync;
end

assign aio[6:5]  = r_out;
assign aio[8:7]  = g_out;
assign aio[10:9] = b_out;
assign aio[11]   = v_sync_out;
assign aio[12]   = h_sync_out;


endmodule
