`timescale 1ns / 1ps

//******************************************************************************
//* VGA idzts genertor 640x480 @ 60 Hz felbontshoz.                       *
//******************************************************************************
module vga_timing(
   input  wire       clk,        //25 MHz pixel rajel.
   input  wire       rst,        //Reset jel.
   output reg  [9:0] h_cnt,      //Horizontlis szmll (X koordinta).
   output reg  [9:0] v_cnt,      //Vertiklis szmll (Y koordinta)
   output wire       blank,      //Kiolt jel.
   output reg        h_sync,     //Horizontlis szinkronjel.
   output reg        v_sync      //Vertiklis szinkronjel.
);

//******************************************************************************
//* A 640x480 @ 60 Hz VGA felbonts idztsi paramterei.                     *
//******************************************************************************
//Egy sorban a lthat pixelek szma.
localparam H_VISIBLE     = ...
//A horizontlis szinkron pulzus eltti pixelek szma a nem lthat kptartomnyban.
localparam H_FRONT_PORCH = ...
//A horizontlis szinkron pulzus hossza (pixel).
localparam H_SYNC_PULSE  = ...
//A horizontlis szinkron pulzus utni pixelek szma a nem lthat kptartomnyban.
localparam H_BACK_PORCH  = ...

//Egy kpen a lthat sorok szma.
localparam V_VISIBLE     = ...
//A vertiklis szinkron pulzus eltti sorok szma a nem lthat kptartomnyban.
localparam V_FRONT_PORCH = ...
//A vertiklis szinkron pulzus hossza (sor).
localparam V_SYNC_PULSE  = ...
//A vertiklis szinkron pulzus utni sorok szma a nem lthat kptartomnyban.
localparam V_BACK_PORCH  = ...

//A horizontlis kiolt jel aktivlst ennl a pixelnl engedlyezzk.
localparam H_BLANK_SET   = H_VISIBLE - 1;
//A horizontlis szinkronjel aktivlst ennl a pixelnl engedlyezzk.
localparam H_SYNC_SET    = H_BLANK_SET + H_FRONT_PORCH;
//A horizontlis szinkronjel trlst ennl a pixelnl engedlyezzk.
localparam H_SYNC_CLR    = H_SYNC_SET + H_SYNC_PULSE;
//A horizontlis kiolt jel trlst ennl a pixelnl engedlyezzk.
localparam H_BLANK_CLR   = H_SYNC_CLR + H_BACK_PORCH;

//A vertiklis kiolt jel aktivlst ennek a sornak a vgn engedlyezzk.
localparam V_BLANK_SET   = V_VISIBLE - 1;
//A vertiklis szinkronjel aktivlst ennek a sornak a vgn engedlyezzk.
localparam V_SYNC_SET    = V_BLANK_SET + V_FRONT_PORCH;
//A vertiklis szinkronjel trlst ennek a sornak a vgn engedlyezzk.
localparam V_SYNC_CLR    = V_SYNC_SET + V_SYNC_PULSE;
//A vertiklis kiolt jel trlst ennek a sornak a vgn engedlyezzk.
localparam V_BLANK_CLR   = V_SYNC_CLR + V_BACK_PORCH;


//******************************************************************************
//* Horizontlis (X) s vertiklis (Y) szmllk. A vertiklis szmll rtke *
//* mindig a sorok vgn nvekszik eggyel.                                     *
//******************************************************************************
reg [9:0] h_cnt;
reg [9:0] v_cnt;

always @(posedge clk)
begin
   ...
end

always @(posedge clk)
begin
	...
end


//******************************************************************************
//* A kiolt jeleket elllt flip-flop-ok. A nem lthat kptartomnyban     *
//* mindig fekete sznt kell kiadni, ezt jelzi a kiolt jel.                   *
//******************************************************************************
reg  h_blank;
reg  v_blank;
wire blank = h_blank | v_blank;

always @(posedge clk)
begin
	...
end

always @(posedge clk)
begin
	...
end


//******************************************************************************
//* A szinkron jeleket elllt flip-flop-ok. A monitor a horizontlis s     *
//* a vertiklis szinkron jelek idztse alapjn tudja megllaptani a        *
//* felbontst s tud szinkronizlni a pixel adatokra (sznjelek).             *
//******************************************************************************
reg h_sync;
reg v_sync;

always @(posedge clk)
begin
	...
end

always @(posedge clk)
begin
	...
end


endmodule
