`timescale 1ns / 1ps

module vga_top(
   input  wire        clk16M,
   input  wire        rstbt,
	input  wire [1:0]  bt,
   output wire [12:5] aio
);

//******************************************************************************
//* rajel genertor. A 16 MHz bejv rajelbl ellltja a 25 MHz-es pixel   *
//* rajelet.                                                                  *
//******************************************************************************
wire clk;
wire rst;

clock_generator #(
	//f_clk_out = f_clk_in * F_CLKIN_MUL / F_CLKIN_DIV.
	//A bemen rajel frekvencija Hz-ben.
   .CLKIN_FREQ_HZ(16000000.0),
	//Szorz a bemen rajel frekvencijhoz (2 - 32).
   .F_CLKIN_MUL(25),
	//Oszt a bemen rajel frekvencijhoz (1 - 32).
   .F_CLKIN_DIV(16)
) clkgen (
	//Bemen rajel.
   .clk_in(clk16M),
	//Bemen reset jel.
   .rst_in(rstbt),
	//Kimen rajel.
   .clk_out(clk),
	//Kimen reset jel.
   .rst_out(rst)
);


//******************************************************************************
//* VGA idzts genertor 640x480 @ 60 Hz felbontshoz.                       *
//******************************************************************************
wire [9:0] h_cnt;
wire [9:0] v_cnt;
wire       blank;
wire       h_sync;
wire       v_sync;

vga_timing vga_timing(
   .clk(clk),           //25 MHz pixel rajel.
   .rst(rst),           //Reset jel.
   .h_cnt(h_cnt),       //Horizontlis szmll (X koordinta).
   .v_cnt(v_cnt),       //Vertiklis szmll (Y koordinta)
   .blank(blank),       //Kiolt jel.
   .h_sync(h_sync),     //Horizontlis szinkronjel.
   .v_sync(v_sync)      //Vertiklis szinkronjel.
);


//******************************************************************************
//* A httr adatokat trol ROM. 1 rajelperidust ksleltet!                 *
//*                                                                            *
//* A httr 32x16 csempbl ll, ebbl 20x15 lthat a kpen. A csempk       *
//* 32x32 pixel mretek a kpernyn. A cmzsnl a szmll bitek ennek      *
//* megfelelen vlasztandk ki.                                               *    
//******************************************************************************
(* rom_style = "block" *)
reg  [4:0] bg_rom [511:0];
wire [8:0] bg_rom_addr = {..., ...};
reg  [4:0] bg_rom_data;

initial
begin
   $readmemh("src/bg_data.txt", bg_rom, 0, 511);
end

always @(posedge clk)
begin
   bg_rom_data <= ...
end

//A ROM ksleltetse miatt a ksbbiekben hasznlt jeleket is ksleltetni kell.
reg [9:0] h_cnt_delay1;
reg [9:0] v_cnt_delay1;
reg       blank_delay1;
reg       h_sync_delay1;
reg       v_sync_delay1;

always @(posedge clk)
begin
   ...
end


//******************************************************************************
//* Felfut l detektlsa a vertiklis szinkronjelen. Ez 60 Hz-es temez     *
//* jelknt hasznlhat fel.                                                   *
//******************************************************************************
reg  [1:0] v_sync_samples;
wire       v_sync_rising = (v_sync_samples == ...);

always @(posedge clk)
begin
	if (rst)
		v_sync_samples <= 2'b11;
	else
		v_sync_samples <= {v_sync_samples[0], v_sync};
end


//******************************************************************************
//* A nyomgombok szinkronizlsa a 25 MHz rendszerrajelhez s                *
//* pergsmentestse.                                                         *
//******************************************************************************
reg [1:0] bt0_samples;
reg [1:0] bt1_samples;

always @(posedge clk)
begin
	if (rst)
	begin
		bt0_samples <= 2'b00;
		bt1_samples <= 2'b00;
	end
	else
	begin
		bt0_samples <= {bt0_samples[0], bt[0]};
		bt1_samples <= {bt1_samples[0], bt[1]};
	end
end

reg  bt0_pressed;
reg  bt1_pressed;
wire btn_pressed = bt0_pressed | bt1_pressed;

always @(posedge clk)
begin
   if (rst)
   begin
      bt0_pressed <= 1'b0;
      bt1_pressed <= 1'b0;
   end
   else
      if (v_sync_rising)
      begin
         bt0_pressed <= bt0_samples[1];
         bt1_pressed <= bt1_samples[1];
      end
end


//******************************************************************************
//* A sprite kivlaszt jel ellltsa.                                       *
//******************************************************************************
/*
//A sprite koordinti (az Y koordinta konstans).
wire [9:0] sprite_x = 32*10;
wire [9:0] sprite_y = 32*11;

//always @(posedge clk)
//begin
//	if (rst)
//		sprite_x <= 32*10;
//	else
//		if (btn_pressed && v_sync_rising)
//			if (... && (... < ...))
//				sprite_x <= ...;
//			else
//				if (... && (... > ...))
//					sprite_x <= ...;
//end

//Kpezzk a szmllk s a sprite koordintk klnbsgt. A kivonnak
//nincs kimen tvitel bitje, ezrt eggyel nagyobb bitszmot hasznlunk.
wire [10:0] x_diff = {1'b0, h_cnt_delay1} - {1'b0, sprite_x};
wire [10:0] y_diff = {1'b0, v_cnt_delay1} - {1'b0, sprite_y};

//A sprite kivlaszt jel. A sprite pixel akkor jelentend meg, ha a
//fenti klnbsg koordintk a sprite 32x64 pixel mret hatrol
//tglalapjn bell vannak. Azaz nincs tvitel (a lagnagyobb helyirtk
//bit 0) s a mretnek megfelel szm fels bitek is nullk.
wire sprite_sel = (...) & (...);
*/


//******************************************************************************
//* A sprite mozgsi irnyt jelz jel. Ha a sprite balra mozog, akkor         *
//* tkrzzk a csempit az Y tengelyre.                                       *
//******************************************************************************
/*
reg sprite_dir; 	//0: jobbra, 1: balra

always @(posedge clk)
begin
	if (rst)
		sprite_dir <= 1'b0;
	else
		if (v_sync_rising)
			if (...)
				sprite_dir <= ...;
			else
				if (...)
					sprite_dir <= ...;
end
*/


//******************************************************************************
//* A sprite animcis fzisainak szmllja. Ha nincs mozgats, akkor a 0.    *
//* fzist jelentjk meg, egybknt ismtlden az 1., 2. s 3. fzisokat.    *
//******************************************************************************
/*
reg [4:0] sprite_frame;

always @(posedge clk)
begin
	if (rst)
		sprite_frame <= ...;
	else
		if (v_sync_rising)
			if (...)
				sprite_frame <= ...;
			else
				if ((sprite_frame == ...) || (sprite_frame == ...))
					sprite_frame <= ...;
				else
					sprite_frame <= ...;
end
*/


//******************************************************************************
//* A csempe adatokat trol ROM. 1 rajelperidust ksleltet!                 *
//*                                                                            *
//* Egy csempe 16x16 pixel a ROM-ban, melyet a kpernyn 32x32 pixel mretre  *
//* nagytunk. Az els 24 csempe tartozik a httrhez, a maradk 8 pedig a     *
//* a spritehoz (4 animcis fzis). A cmzsnl a szmll bitek ennek        *
//* megfelelen vlasztandk ki.                                               * 
//******************************************************************************
(* rom_style = "block" *)
reg  [5:0]  tile_rom [8191:0];
wire [12:0] tile_rom_addr = {..., ..., ...};
reg  [5:0]  tile_rom_data;
//wire [12:0] sprite_addr   = {..., ..., ..., ...};
//reg  [5:0]  sprite_data;

initial
begin
   $readmemh("src/tile_data.txt", tile_rom, 0, 8191);
end

always @(posedge clk)
begin
   tile_rom_data <= tile_rom[tile_rom_addr];
	//sprite_data   <= tile_rom[sprite_addr];
end

//A ROM ksleltetse miatt a ksbbiekben hasznlt jeleket is ksleltetni kell.
reg blank_delay2;
reg h_sync_delay2;
reg v_sync_delay2;
//reg sprite_sel_delay;

always @(posedge clk)
begin
   ...
end


//******************************************************************************
//* A kimenetek meghajtsa. Ha a sznjeleket regiszterezzk, akkor a szinkron  *
//* jeleket is ksleltetni kell 1 rajel peridussal. Ha a kiolt jel aktv,   *
//* akkor a sznjeleket 0-val hajtsuk meg (fekete szn).                       *
//*                                                                            *
//* A sprite esetn a fehr szn a transzparens.                               *
//******************************************************************************
reg [1:0] r_out, g_out, b_out;
reg       h_sync_out, v_sync_out;

always @(posedge clk)
begin
   if (...)
      {b_out, g_out, r_out} <= 6'b00_00_00;
   else
//      if (...)
//         {b_out, g_out, r_out} <= sprite_data;
//      else
         {b_out, g_out, r_out} <= tile_rom_data;
	
	h_sync_out <= h_sync_delay2;
	v_sync_out <= v_sync_delay2;
end

assign aio[6:5]  = r_out;
assign aio[8:7]  = g_out;
assign aio[10:9] = b_out;
assign aio[11]   = v_sync_out;
assign aio[12]   = h_sync_out;


endmodule
